==> bfcc-0.1.tar.gz: bfcc/symtable.c <==
/*
    Copyright (c) 2002 Florian Wesch <fw@dividuum.de>. All Rights Reserved.     
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/    


#include <string.h>
#include <stdlib.h>
#include "symtable.h"
#include "common.h"

SymbolTable *STInit() {
    SymbolTable *st;

    st = (SymbolTable*)malloc(sizeof(SymbolTable));

    if (st == NULL) 
        fatalerror("kein Speicher fr SymbolTable?");
    
    st->first = NULL;
    st->lastmempos = 0;
    return st;
}

void NewSymbol(SymbolTable *st, char *name, int size) {
    Symbol *new  = (Symbol*)malloc(sizeof(Symbol));

    if (new == NULL)
        fatalerror("kein Speicher fr Symbol?");

    new->next = st->first;
    new->name = strdup(name);
    new->mempos = INVALIDADDR;
    new->size = size;

    st->first = new;
}

Symbol *GetSymbol(SymbolTable *st, char *name) {
    Symbol *cur  = st->first;

    while (cur) {
        if (strcmp(name, cur->name) == 0) // Symbol gefunden
            return cur;
        cur = cur->next;
    }
    return NULL;
}

void SymbolGrowSize(SymbolTable *st, char *name, int possible_new_size) {
    Symbol *cur  = st->first;

    while (cur) {
        if (strcmp(name, cur->name) == 0) { // Symbol gefunden
            if (possible_new_size >= cur->size)
                cur->size = possible_new_size;
            return;
        }
        cur = cur->next;
    }
}

void STAssignLocation(SymbolTable *st) {
    int     pos  = 0;
    Symbol *cur  = st->first;
    while (cur) {
        cur->mempos     = pos;
        pos            += cur->size;
        st->lastmempos += cur->size;
        cur             = cur->next;
    }
}

BFADDR GetSymLoc(SymbolTable *st, char *name) {
    Symbol *cur  = st->first;
    while (cur) {
        if (strcmp(name, cur->name) == 0) // Symbol gefunden
            return cur->mempos;
        cur = cur->next;
    }
	return INVALIDADDR;
}


void STDump(SymbolTable *st) {
    Symbol *cur  = st->first;
    info("--[ Symtable ]----------\n");
    while (cur) {
        info("@%03d (size %03d): %s\n", cur->mempos, cur->size, cur->name);
        cur = cur->next;
    }
    //info("lastmempos %d\n", st->lastmempos);
    info("------------------------\n");
}

==> bfcc-0.2.tar.gz: bfcc/symtable.c <==
/*
    Copyright (c) 2002 Florian Wesch <fw@dividuum.de>. All Rights Reserved.     
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/    


#include <string.h>
#include <stdlib.h>
#include "symtable.h"
#include "common.h"

SymbolTable *STInit() {
    SymbolTable *st;

    st = (SymbolTable*)malloc(sizeof(SymbolTable));

    if (st == NULL) 
        fatalerror("kein Speicher fr SymbolTable?");
    
    st->first = NULL;
    st->lastmempos = 0;
    return st;
}

void SymSetSize(Symbol *s, int size) {
    s->numelements = size;
    s->memsize  = (size > 1) ? 3+2*size : size;
}

void NewSymbol(SymbolTable *st, char *name, int size) {
    Symbol *new  = (Symbol*)malloc(sizeof(Symbol));

    if (new == NULL)
        fatalerror("kein Speicher fr Symbol?");

    new->next = st->first;
    st->first = new;

    new->name = strdup(name);
    new->mempos  = INVALIDADDR;
    SymSetSize(new, size);
}

Symbol *GetSymbol(SymbolTable *st, char *name) {
    Symbol *cur  = st->first;

    while (cur) {
        if (strcmp(name, cur->name) == 0) // Symbol gefunden
            return cur;
        cur = cur->next;
    }
    return NULL;
}

int SymbolGrowSize(SymbolTable *st, char *name, int possible_new_size) {
    Symbol *cur  = st->first;

    while (cur) {
        if (strcmp(name, cur->name) == 0) { // Symbol gefunden
            if (possible_new_size > cur->numelements) {
                SymSetSize(cur, possible_new_size);
                return 1;
            }
            return 0;
        }
        cur = cur->next;
    }
    return 0;
}

void STAssignLocation(SymbolTable *st) {
    int     pos  = 0;
    Symbol *cur  = st->first;
    while (cur) {
        cur->mempos     = pos;
        pos            += cur->memsize;
        st->lastmempos += cur->memsize;
        cur             = cur->next;
    }
}

Symbol *GetSym(SymbolTable *st, char *name) {
    Symbol *cur  = st->first;
    while (cur) {
        if (strcmp(name, cur->name) == 0) // Symbol gefunden
            return cur;
        cur = cur->next;
    }
    return NULL;
}


void STDump(SymbolTable *st) {
    Symbol *cur  = st->first;
    info("--[ Symtable ]----------\n");
    while (cur) {
        info("@%03d %c (memsize %03d): %s\n", cur->mempos, SymIsArray(cur) ? 'A' : 'V', cur->memsize, cur->name);
        cur = cur->next;
    }
    //info("lastmempos %d\n", st->lastmempos);
    info("------------------------\n");
}
